/*----------------------------------------------------------------------- 
 * 
 * File Name: ArrayFactories.c
 * 
 * Revision: $Id: ArrayFactories.m4,v 1.1 2000/04/17 23:22:42 jolien Exp $
 * 
 *----------------------------------------------------------------------- 
 * 
 * SYNOPSIS 
 * void TYPECODECreateArray( Status *, TYPEArray **array, UINT4Vector *dimlength );
 * void TYPECODEDestroyArray( Status *, TYPEArray **array);
 * 
 * DESCRIPTION 
 * Create/destroy a TYPEArray object. 
 * 
 * DIAGNOSTICS 
 * Illegal length, array == NULL, *array != NULL, malloc failure
 *
 * CALLS
 * LALMalloc
 * LALFree
 * 
 *-----------------------------------------------------------------------
 */

#include <string.h>
#include "LALStdlib.h"
#include "AVFactories.h"

NRCSID( ARRAYFACTORIESC, "$Id: ArrayFactories.m4,v 1.1 2000/04/17 23:22:42 jolien Exp $" );

















void ZCreateArray ( Status *status, COMPLEX16Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "ZCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( COMPLEX16Array * ) LALMalloc ( sizeof( COMPLEX16Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( COMPLEX16 * ) LALMalloc( arrayDataSize*sizeof( COMPLEX16 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void ZDestroyArray ( Status *status, COMPLEX16Array **array )
{
  INITSTATUS (status, "ZDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void CCreateArray ( Status *status, COMPLEX8Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "CCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( COMPLEX8Array * ) LALMalloc ( sizeof( COMPLEX8Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( COMPLEX8 * ) LALMalloc( arrayDataSize*sizeof( COMPLEX8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void CDestroyArray ( Status *status, COMPLEX8Array **array )
{
  INITSTATUS (status, "CDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void DCreateArray ( Status *status, REAL8Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "DCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL8Array * ) LALMalloc ( sizeof( REAL8Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL8 * ) LALMalloc( arrayDataSize*sizeof( REAL8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void DDestroyArray ( Status *status, REAL8Array **array )
{
  INITSTATUS (status, "DDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void SCreateArray ( Status *status, REAL4Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "SCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL4Array * ) LALMalloc ( sizeof( REAL4Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL4 * ) LALMalloc( arrayDataSize*sizeof( REAL4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void SDestroyArray ( Status *status, REAL4Array **array )
{
  INITSTATUS (status, "SDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void I2CreateArray ( Status *status, INT2Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "I2CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT2Array * ) LALMalloc ( sizeof( INT2Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT2 * ) LALMalloc( arrayDataSize*sizeof( INT2 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void I2DestroyArray ( Status *status, INT2Array **array )
{
  INITSTATUS (status, "I2DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void I4CreateArray ( Status *status, INT4Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "I4CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT4Array * ) LALMalloc ( sizeof( INT4Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT4 * ) LALMalloc( arrayDataSize*sizeof( INT4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void I4DestroyArray ( Status *status, INT4Array **array )
{
  INITSTATUS (status, "I4DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void I8CreateArray ( Status *status, INT8Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "I8CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT8Array * ) LALMalloc ( sizeof( INT8Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT8 * ) LALMalloc( arrayDataSize*sizeof( INT8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void I8DestroyArray ( Status *status, INT8Array **array )
{
  INITSTATUS (status, "I8DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void U2CreateArray ( Status *status, UINT2Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "U2CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT2Array * ) LALMalloc ( sizeof( UINT2Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT2 * ) LALMalloc( arrayDataSize*sizeof( UINT2 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void U2DestroyArray ( Status *status, UINT2Array **array )
{
  INITSTATUS (status, "U2DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void U4CreateArray ( Status *status, UINT4Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "U4CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT4Array * ) LALMalloc ( sizeof( UINT4Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT4 * ) LALMalloc( arrayDataSize*sizeof( UINT4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void U4DestroyArray ( Status *status, UINT4Array **array )
{
  INITSTATUS (status, "U4DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void U8CreateArray ( Status *status, UINT8Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "U8CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT8Array * ) LALMalloc ( sizeof( UINT8Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT8 * ) LALMalloc( arrayDataSize*sizeof( UINT8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void U8DestroyArray ( Status *status, UINT8Array **array )
{
  INITSTATUS (status, "U8DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


















void CreateArray ( Status *status, REAL4Array **array, UINT4Vector *dimLength ) 
{
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (!*array,           status, CREATEARRAY_EUPTR, CREATEARRAY_MSGEUPTR);
  ASSERT (dimLength,         status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->data,   status, CREATEARRAY_EVPTR, CREATEARRAY_MSGEVPTR);
  ASSERT (dimLength->length, status,
          CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, CREATEARRAY_ELENGTH, CREATEARRAY_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL4Array * ) LALMalloc ( sizeof( REAL4Array ) );
  ASSERT (*array, status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  U4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL4 * ) LALMalloc( arrayDataSize*sizeof( REAL4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    U4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, CREATEARRAY_EMALLOC, CREATEARRAY_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
















void DestroyArray ( Status *status, REAL4Array **array )
{
  INITSTATUS (status, "DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, DESTROYARRAY_EVPTR, DESTROYARRAY_MSGEVPTR);
  ASSERT (*array,         status, DESTROYARRAY_EUPTR, DESTROYARRAY_MSGEUPTR);
  ASSERT ((*array)->data, status, DESTROYARRAY_EDPTR, DESTROYARRAY_MSGEDPTR);

  /* Free allocated storage */

  U4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

