/*----------------------------------------------------------------------- 
 * 
 * File Name: VectorSequenceFactoriesTest.c
 * 
 * Author: Finn, L. S. 
 * 
 * Revision: $Id: VectorSequenceFactoriesTest.m4,v 1.1 2000/04/17 23:21:08 jolien Exp $
 * 
 *----------------------------------------------------------------------- 
 * 
 * DESCRIPTION 
 *   Test suite for vector factories
 * 
 * DIAGNOSTICS
 *   Writes PASS, FAIL to stdout as tests are passed or failed. 
 *   Returns 0 if all tests passed and -1 if any test failed. 
 * 
 * CALLS
 *   TYPECODECreateVectorSequence
 *   TYPECODEDestroyVectorSequence
 * 
 * NOTES
 *   PROGNAME can't test LALMalloc failure without messing with
 *   LALMalloc
 * 
 *-----------------------------------------------------------------------
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "LALConfig.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "LALStdlib.h"
#include "SeqFactories.h"

#define CODES_(x) #x
#define CODES(x) CODES_(x)

NRCSID( MAIN, "$Id: VectorSequenceFactoriesTest.m4,v 1.1 2000/04/17 23:21:08 jolien Exp $" );

extern char *optarg;
extern int   optind;

int debuglevel = 1;
int verbose    = 0;

static void
Usage (const char *program, int exitflag);

static void
ParseOptions (int argc, char *argv[]);

static void
TestStatus (Status *status, const char *expectedCodes, int exitCode);

static void
ClearStatus (Status *status);




















void ZVectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static COMPLEX16VectorSequence  *sequence;
  static COMPLEX16VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  ZCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( COMPLEX16 ) );

  ZDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  ZCreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  ZCreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  ZCreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  ZDestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  ZCreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  ZDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  ZCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  ZDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of ZCreateVectorSequence and ZDestroyVectorSequence \n" );
          
  return;
}





















void CVectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static COMPLEX8VectorSequence  *sequence;
  static COMPLEX8VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( COMPLEX8 ) );

  CDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CCreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  CCreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  CCreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  CDestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  CCreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  CDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  CCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  CDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CCreateVectorSequence and CDestroyVectorSequence \n" );
          
  return;
}





















void DVectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static REAL8VectorSequence  *sequence;
  static REAL8VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  DCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( REAL8 ) );

  DDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  DCreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  DCreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  DCreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  DDestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  DCreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  DDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  DCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  DDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of DCreateVectorSequence and DDestroyVectorSequence \n" );
          
  return;
}





















void SVectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static REAL4VectorSequence  *sequence;
  static REAL4VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  SCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( REAL4 ) );

  SDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  SCreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  SCreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  SCreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  SDestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  SCreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  SDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  SCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  SDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of SCreateVectorSequence and SDestroyVectorSequence \n" );
          
  return;
}





















void I2VectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static INT2VectorSequence  *sequence;
  static INT2VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I2CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( INT2 ) );

  I2DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I2CreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  I2CreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  I2CreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  I2DestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  I2CreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  I2DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  I2CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  I2DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I2CreateVectorSequence and I2DestroyVectorSequence \n" );
          
  return;
}





















void I4VectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static INT4VectorSequence  *sequence;
  static INT4VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I4CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( INT4 ) );

  I4DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I4CreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  I4CreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  I4CreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  I4DestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  I4CreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  I4DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  I4CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  I4DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I4CreateVectorSequence and I4DestroyVectorSequence \n" );
          
  return;
}





















void I8VectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static INT8VectorSequence  *sequence;
  static INT8VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I8CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( INT8 ) );

  I8DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I8CreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  I8CreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  I8CreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  I8DestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  I8CreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  I8DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  I8CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  I8DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I8CreateVectorSequence and I8DestroyVectorSequence \n" );
          
  return;
}





















void U2VectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static UINT2VectorSequence  *sequence;
  static UINT2VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U2CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( UINT2 ) );

  U2DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U2CreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  U2CreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  U2CreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  U2DestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  U2CreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  U2DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  U2CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  U2DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U2CreateVectorSequence and U2DestroyVectorSequence \n" );
          
  return;
}





















void U4VectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static UINT4VectorSequence  *sequence;
  static UINT4VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U4CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( UINT4 ) );

  U4DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U4CreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  U4CreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  U4CreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  U4DestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  U4CreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  U4DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  U4CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  U4DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U4CreateVectorSequence and U4DestroyVectorSequence \n" );
          
  return;
}





















void U8VectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static UINT8VectorSequence  *sequence;
  static UINT8VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U8CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( UINT8 ) );

  U8DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U8CreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  U8CreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  U8CreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  U8DestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  U8CreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  U8DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  U8CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  U8DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U8CreateVectorSequence and U8DestroyVectorSequence \n" );
          
  return;
}





















void CHARVectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static CHARVectorSequence  *sequence;
  static CHARVectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CHARCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( CHAR ) );

  CHARDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CHARCreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  CHARCreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  CHARCreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  CHARDestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  CHARCreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  CHARDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  CHARCreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  CHARDestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CHARCreateVectorSequence and CHARDestroyVectorSequence \n" );
          
  return;
}





















void VectorSequenceFactoriesTest ( void )
{
  CreateVectorSequenceIn input   = { 2, 8 };
  CreateVectorSequenceIn badslen = { 0, 8 };
  CreateVectorSequenceIn badvlen = { 2, 0 };
  static Status  status;
  static REAL4VectorSequence  *sequence;
  static REAL4VectorSequence   sstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( sequence->data, 0, input.length*input.vectorLength*sizeof( REAL4 ) );

  DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CreateVectorSequence ( &status, &sequence, &badslen );
  TestStatus( &status, CODES( CREATEVECSEQ_ESLENGTH ), 1 );

  CreateVectorSequence ( &status, &sequence, &badvlen );
  TestStatus( &status, CODES( CREATEVECSEQ_EVLENGTH ), 1 );

  CreateVectorSequence ( &status, &sequence, NULL );
  TestStatus( &status, CODES( CREATEVECSEQ_EINPTR ), 1 );

  DestroyVectorSequence ( &status, NULL );
  TestStatus( &status, CODES( DESTROYVECSEQ_EVPTR ), 1 );

  CreateVectorSequence ( &status, NULL, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EVPTR ), 1 );

  DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EUPTR ), 1 );

  sequence = &sstore;
  CreateVectorSequence ( &status, &sequence, &input );
  TestStatus( &status, CODES( CREATEVECSEQ_EUPTR ), 1 );

  DestroyVectorSequence ( &status, &sequence );
  TestStatus( &status, CODES( DESTROYVECSEQ_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CreateVectorSequence and DestroyVectorSequence \n" );
          
  return;
}


int main( int argc, char *argv[] )
{
  ParseOptions( argc, argv );

  VectorSequenceFactoriesTest();
  ZVectorSequenceFactoriesTest();
  CVectorSequenceFactoriesTest();
  DVectorSequenceFactoriesTest();
  SVectorSequenceFactoriesTest();
  I2VectorSequenceFactoriesTest();
  I4VectorSequenceFactoriesTest();
  I8VectorSequenceFactoriesTest();
  U2VectorSequenceFactoriesTest();
  U4VectorSequenceFactoriesTest();
  U8VectorSequenceFactoriesTest();
  CHARVectorSequenceFactoriesTest();

  return 0;
}


/*
 * TestStatus ()
 *
 * Routine to check that the status code status->statusCode agrees with one of
 * the codes specified in the space-delimited string ignored; if not,
 * exit to the system with code exitcode.
 *
 */
static void
TestStatus (Status *status, const char *ignored, int exitcode)
{
  char  str[64];
  char *tok;

  if (verbose)
  {
    REPORTSTATUS (status);
  }

  if (strncpy (str, ignored, sizeof (str)))
  {
    if ((tok = strtok (str, " ")))
    {
      do
      {
        if (status->statusCode == atoi (tok))
        {
          return;
        }
      }
      while ((tok = strtok (NULL, " ")));
    }
    else
    {
      if (status->statusCode == atoi (tok))
      {
        return;
      }
    }
  }

  fprintf (stderr, "\nExiting to system with code %d\n", exitcode);
  exit (exitcode);
}


/*
 *
 * ClearStatus ()
 *
 * Recursively applies DETATCHSTATUSPTR() to status structure to destroy
 * linked list of statuses.
 *
 */
void
ClearStatus (Status *status)
{
  if (status->statusPtr)
  {
    ClearStatus      (status->statusPtr);
    DETATCHSTATUSPTR (status);
  }
}


/*
 * Usage ()
 *
 * Prints a usage message for program program and exits with code exitcode.
 *
 */
static void
Usage (const char *program, int exitcode)
{
  fprintf (stderr, "Usage: %s [options]\n", program);
  fprintf (stderr, "Options:\n");
  fprintf (stderr, "  -h         print this message\n");
  fprintf (stderr, "  -q         quiet: run silently\n");
  fprintf (stderr, "  -v         verbose: print extra information\n");
  fprintf (stderr, "  -d level   set debuglevel to level\n");
  exit (exitcode);
}


/*
 * ParseOptions ()
 *
 * Parses the argc - 1 option strings in argv[].
 *
 */
static void
ParseOptions (int argc, char *argv[])
{
  while (1)
  {
    int c = -1;

    c = getopt (argc, argv, "hqvd:");
    if (c == -1)
    {
      break;
    }

    switch (c)
    {
      case 'd': /* set debug level */
        debuglevel = atoi (optarg);
        break;

      case 'v': /* verbose */
        ++verbose;
        break;

      case 'q': /* quiet: run silently (ignore error messages) */
        freopen ("/dev/null", "w", stderr);
        freopen ("/dev/null", "w", stdout);
        break;

      case 'h':
        Usage (argv[0], 0);
        break;

      default:
        Usage (argv[0], 1);
    }

  }

  if (optind < argc)
  {
    Usage (argv[0], 1);
  }

  return;
}

